@@ -14,9 +14,15 @@ from cryptography.hazmat.primitives.ciphers.aead import ChaCha20Poly1305
 import curve25519
 import ed25519
 
+from pyhap.const import (
+    CATEGORY_BRIDGE,
+    HAP_REPR_CHARS,
+    HAP_REPR_STATUS,
+    HAP_SERVER_STATUS,
+)
 import pyhap.tlv as tlv
 from pyhap.util import long_to_bytes
-from pyhap.const import CATEGORY_BRIDGE
+
 from .hap_crypto import hap_hkdf, pad_tls_nonce
 
 SNAPSHOT_TIMEOUT = 10
@@ -73,31 +79,11 @@ class HAP_TLV_TAGS:
     PERMISSIONS = b"\x0B"
 
 
-# Status codes for underlying HAP calls
-class HAP_SERVER_STATUS:
-    SUCCESS = 0
-    INSUFFICIENT_PRIVILEGES = -70401
-    SERVICE_COMMUNICATION_FAILURE = -70402
-    RESOURCE_BUSY = -70403
-    READ_ONLY_CHARACTERISTIC = -70404
-    WRITE_ONLY_CHARACTERISTIC = -70405
-    NOTIFICATION_NOT_SUPPORTED = -70406
-    OUT_OF_RESOURCE = -70407
-    OPERATION_TIMED_OUT = -70408
-    RESOURCE_DOES_NOT_EXIST = -70409
-    INVALID_VALUE_IN_REQUEST = -70410
-    INSUFFICIENT_AUTHORIZATION = -70411
-
-
 class HAP_PERMISSIONS:
     USER = b"\x00"
     ADMIN = b"\x01"
 
 
-class TimeoutException(Exception):
-    pass
-
-
 class UnprivilegedRequestException(Exception):
     pass
 
@@ -187,13 +173,13 @@ class HAPServerHandler:
             "pre_session_key": pre_session_key,
         }
 
-    def send_response(self, code, message=None):
+    def send_response(self, http_status):
         """Add the response header to the headers buffer and log the
         response code.
         Does not add Server or Date
         """
-        self.response.status_code = int(code)
-        self.response.reason = message or "OK"
+        self.response.status_code = http_status.value
+        self.response.reason = http_status.phrase
 
     def send_header(self, header, value):
         """Add the response header to the headers buffer."""
@@ -221,21 +207,19 @@ class HAPServerHandler:
         )
 
         path = urlparse(self.path).path
-        assert path in self.HANDLERS[self.command]
         try:
             getattr(self, self.HANDLERS[self.command][path])()
         except UnprivilegedRequestException:
             self.send_response_with_status(
-                401, HAP_SERVER_STATUS.INSUFFICIENT_PRIVILEGES
+                HTTPStatus.UNAUTHORIZED, HAP_SERVER_STATUS.INSUFFICIENT_PRIVILEGES
             )
-        except TimeoutException:
-            self.send_response_with_status(500, HAP_SERVER_STATUS.OPERATION_TIMED_OUT)
         except Exception:  # pylint: disable=broad-except
             logger.exception(
                 "%s: Failed to process request for: %s", self.client_address, path
             )
             self.send_response_with_status(
-                500, HAP_SERVER_STATUS.SERVICE_COMMUNICATION_FAILURE
+                HTTPStatus.INTERNAL_SERVER_ERROR,
+                HAP_SERVER_STATUS.SERVICE_COMMUNICATION_FAILURE,
             )
 
         self.response = None
@@ -245,7 +229,8 @@ class HAPServerHandler:
         """Generate a generic failure response."""
         self.response = HAPResponse()
         self.send_response_with_status(
-            500, HAP_SERVER_STATUS.SERVICE_COMMUNICATION_FAILURE
+            HTTPStatus.INTERNAL_SERVER_ERROR,
+            HAP_SERVER_STATUS.SERVICE_COMMUNICATION_FAILURE,
         )
         response = self.response
         self.response = None
@@ -343,10 +328,13 @@ class HAPServerHandler:
         )
 
         cipher = ChaCha20Poly1305(hkdf_enc_key)
-        decrypted_data = cipher.decrypt(
-            self.PAIRING_3_NONCE, bytes(encrypted_data), b""
-        )
-        assert decrypted_data is not None
+        try:
+            decrypted_data = cipher.decrypt(
+                self.PAIRING_3_NONCE, bytes(encrypted_data), b""
+            )
+        except InvalidTag:
+            self._send_authentication_error_tlv_response(HAP_TLV_STATES.M6)
+            return
 
         dec_tlv_objects = tlv.decode(bytes(decrypted_data))
         client_username = dec_tlv_objects[HAP_TLV_TAGS.USERNAME]
@@ -425,7 +413,8 @@ class HAPServerHandler:
 
         if not should_confirm:
             self.send_response_with_status(
-                500, HAP_SERVER_STATUS.INVALID_VALUE_IN_REQUEST
+                HTTPStatus.INTERNAL_SERVER_ERROR,
+                HAP_SERVER_STATUS.INVALID_VALUE_IN_REQUEST,
             )
             return
 
@@ -518,8 +507,6 @@ class HAPServerHandler:
             self._send_authentication_error_tlv_response(HAP_TLV_STATES.M4)
             return
 
-        assert decrypted_data is not None  # TODO:
-
         dec_tlv_objects = tlv.decode(bytes(decrypted_data))
         client_username = dec_tlv_objects[HAP_TLV_TAGS.USERNAME]
         material = (
@@ -567,7 +554,7 @@ class HAPServerHandler:
 
         hap_rep = self.accessory_handler.get_accessories()
         data = json.dumps(hap_rep).encode("utf-8")
-        self.send_response(200)
+        self.send_response(HTTPStatus.OK)
         self.send_header("Content-Type", self.JSON_RESPONSE_TYPE)
         self.end_response(data)
 
@@ -578,10 +565,22 @@ class HAPServerHandler:
 
         # Check that char exists and ...
         params = parse_qs(urlparse(self.path).query)
-        chars = self.accessory_handler.get_characteristics(params["id"][0].split(","))
+        response = self.accessory_handler.get_characteristics(
+            params["id"][0].split(",")
+        )
+        chars = response[HAP_REPR_CHARS]
 
-        data = json.dumps(chars).encode("utf-8")
-        self.send_response(207)
+        had_failure = any(
+            result[HAP_REPR_STATUS] != HAP_SERVER_STATUS.SUCCESS for result in chars
+        )
+        if had_failure:
+            self.send_response(HTTPStatus.MULTI_STATUS)
+        else:
+            self.send_response(HTTPStatus.OK)
+            for result in chars:
+                del result[HAP_REPR_STATUS]
+
+        data = json.dumps(response).encode("utf-8")
         self.send_header("Content-Type", self.JSON_RESPONSE_TYPE)
         self.end_response(data)
 
@@ -606,7 +605,7 @@ class HAPServerHandler:
             self.send_response(HTTPStatus.NO_CONTENT)
             return
 
-        self.send_response(207)
+        self.send_response(HTTPStatus.MULTI_STATUS)
         self.send_header("Content-Type", self.JSON_RESPONSE_TYPE)
         self.end_response(json.dumps(response).encode("utf-8"))
 
@@ -696,7 +695,7 @@ class HAPServerHandler:
 
     def _send_tlv_pairing_response(self, data):
         """Send a TLV encoded pairing response."""
-        self.send_response(200)
+        self.send_response(HTTPStatus.OK)
         self.send_header("Content-Type", self.PAIRING_RESPONSE_TYPE)
         self.end_response(data)
 
@@ -725,6 +724,6 @@ class HAPServerHandler:
             )
 
         task = asyncio.ensure_future(asyncio.wait_for(coro, SNAPSHOT_TIMEOUT))
-        self.send_response(200)
+        self.send_response(HTTPStatus.OK)
         self.send_header("Content-Type", "image/jpeg")
         self.response.task = task
